<?php
namespace Teknisa\DC\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\DC\Util\Utilities;
use \Zeedhi\DevOps\Util\Utilities AS DevOpsUtilities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Teknisa\DC\Util\Messages;
use Teknisa\DC\Factories;
use Zeedhi\Framework\Controller\Exception;

class ResourceProject {

    protected  $entityManager;
    protected  $environment;
    protected  $directoryFactory;
    /** @var Repositories\ResourceProject $resourceProject*/
    protected $resourceProjectRepository;

    /** @var Repositories\Project */
    protected $projectRepository;

    /** @var Repositories\Resource */
    protected $resourceRepository;

    /** @var Repositories\Repository*/
    protected $repositoryRepository;

    /** @var Repositories\Directory*/
    protected $directoryRepository;

    public function __construct(EntityManager $entityManager, Utilities $environment, Factories\Directory $directoryFactory)
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->directoryFactory = $directoryFactory;
        $this->resourceProjectRepository = $entityManager->getRepository(Utilities::ENTITY_DIR . 'ResourceProject');
        $this->projectRepository = $entityManager->getRepository(Utilities::ENTITY_DIR . 'Project');
        $this->resourceRepository = $entityManager->getRepository(Utilities::ENTITY_DIR . 'Resource');
        $this->repositoryRepository = $entityManager->getRepository(Utilities::ENTITY_DIR . 'Repository');
        $this->directoryRepository = $entityManager->getRepository(Utilities::ENTITY_DIR . 'Directory');

    }

    public function getAllResourceProjectsPaths() {
        $user    = $this->environment->getCurrentUserId();
        $dataSet = $this->resourceProjectRepository->getProjectPaths($user);
        return Utilities::fixParentsForTreeGrid($dataSet);
    }

    public function deleteDirectory($id) {
        $directory = $this->directoryRepository->findOneBy(array('id'=>$id));
        $this->entityManager->remove($directory);
        $this->entityManager->flush();
    }

    public function getAllResourcePaths($projectId){
        $user      = $this->environment->getCurrentUserId();
        $projectId = preg_replace("/[^0-9]/", "",$projectId);
        return $this->resourceProjectRepository->getResourceProjectPaths($user,$projectId);
    }

    public function saveDirectory($repository, $directory, $user = null, $directoryId = null, $linuxDirectory = null, $projectId = null, $replicateConnections= null){
        
        if($user){
            $user = $this->resourceRepository->find($user);
        }else{
            $user = $this->environment->getCurrentUser();
        }
        $repository = $this->repositoryRepository->find($repository);
        $projectPathsIds = $this->resourceProjectRepository->getParentProjectPaths($user, $projectId);
        if($directoryId == null) {
            $directory = $this->directoryFactory->createDirectory($directory, $user, $repository, $linuxDirectory);
            $this->entityManager->persist($directory);
            $this->replicateConnectionsToSubProjects($projectPathsIds, $repository, $directory, $user, $directoryId, $linuxDirectory, $projectId, $replicateConnections);
        } else {
            $entity = $this->directoryRepository->find($directoryId);
            $directory = $this->directoryFactory->updateDirectory($entity, $directory, $user, $repository, $linuxDirectory);
            $this->entityManager->persist($directory);
            $this->replicateConnectionsToSubProjects($projectPathsIds, $repository, $directory, $user, $directoryId, $linuxDirectory, $projectId, $replicateConnections);
        } 
        $this->entityManager->flush();
        return $directory;
        
    }
    
    public function replicateConnectionsToSubProjects($projectPathsIds, $repository, $directory, $user, $directoryId, $linuxDirectory, $projectId, $replicateConnections){
        if($replicateConnections == 'Y'){
            foreach ($projectPathsIds as $projectPathsId) {
                $newProjectId = $projectPathsId['ID'];
                $reposioryFromProjects = $this->resourceProjectRepository->getReposioryFromProject($newProjectId);
                foreach ($reposioryFromProjects as $reposioryFromProject) {
                    if($repository->getUrl() == $reposioryFromProject['URL']){
                        $directoryFromProjects = $this->resourceProjectRepository->getDirectoryFromProject($reposioryFromProject['ID'], $newProjectId, $user->getId());
                        if(!$directoryFromProjects){
                            $this->saveDirectory($reposioryFromProject['ID'], $directory->getLocalDirectory(), $user->getId(), null, $linuxDirectory, $newProjectId, $replicateConnections);
                        }else{
                            $this->saveDirectory($reposioryFromProject['ID'], $directory->getLocalDirectory(), $user->getId(), $directoryFromProjects[0]['ID'], $linuxDirectory, $newProjectId, $replicateConnections);
                        }
                    }
                }
            }
        }
    }

    public function copyDirectories($selectedPaths, $project){
        $connection = $this->entityManager->getConnection();
        try{
            $connection->beginTransaction();
            foreach($selectedPaths as $paths) {
                $directory = $paths['LOCAL_DIRECTORY'];
                $linuxDirectory = $paths['LINUX_DIRECTORY'];
                $repositoryUrl = $paths['REPOSITORY_URL'];
                $this->copyDirectory($repositoryUrl, $directory, $project, $linuxDirectory);
            }
            $connection->commit();
        }catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function copyDirectory($repositoryUrl, $directory, $projectId, $linuxDirectory){
        $user = $this->environment->getCurrentUser();
        /** @var Entities\Repository $repository */
        $project = $this->projectRepository->find($projectId);
        $repository = $this->repositoryRepository->findOneBy(array("url" => $repositoryUrl, "project" => $project));
        if($repository) {
            $oldDirectory = $this->directoryRepository->findOneBy(array("repository" => $repository, "resource" => $user));
            if ($oldDirectory == null) {
                $directory = $this->directoryFactory->createDirectory($directory, $user, $repository, $linuxDirectory);
            } else {
                $directoryId = $oldDirectory->getId();
                /** @var Entities\Directory $entity */
                $entity = $this->directoryRepository->find($directoryId);
                $directory = $this->directoryFactory->updateDirectory($entity, $directory, $user, $repository, $linuxDirectory);
            }
            $this->entityManager->persist($directory);
            $this->entityManager->flush();
        }
        return $directory;
    }

    public function verifyProjectPaths($selectedPaths, $project)
    {
        $failedProjectsPaths = array();
        $successfulProjectPaths = array();
        foreach ($selectedPaths as $paths) {
            $repositoryUrl = $paths['REPOSITORY_URL'];
            $success = $this->verifyDirectory($repositoryUrl, $project);
            if ($success) {
                array_push($successfulProjectPaths, $repositoryUrl);
            } else {
                array_push($failedProjectsPaths, $repositoryUrl);
            }
        }
        return array($successfulProjectPaths, $failedProjectsPaths);
    }

    public function verifyDirectory($repositoryUrl, $projectId) {
        $success = true;
        /** @var Entities\Repository $repository */
        $project = $this->projectRepository->find($projectId);
        $repository = $this->repositoryRepository->findOneBy(array("url" => $repositoryUrl, "project" => $project));
        if($repository == null){
            $success = false;
        }
        return $success;
    }
} 