<?php

namespace Teknisa\DC\Util;

use Zeedhi\Framework\Session\Session;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Utilities {
    const ENTITY_DIR = '\Zeedhi\DevOps\Model\Projectmanagement\Entities\\';
    const CONFIGURATION_ENTITY_DIR = '\Zeedhi\DevOps\Model\Configurationmanager\Entities\\';

    public function __construct(Session $session) {
        $this->session = $session;
    }

    public function getCurrentUserId() {
        return $this->session->get('resourceId');
    }

    public function getCurrentOrganizationId() {
        return $this->session->get('organizationId');
    }

    public static function getEntityManager(){
        return InstanceManager::getInstance()->getService('entityManager');
    }

    /**
     * @return \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource
     */
    public function getCurrentUser() {
        $entityManager = self::getEntityManager();
        return $entityManager->find(self::ENTITY_DIR . 'Resource', $this->getCurrentUserId());
    }

    public static function fixParentsForTreeGrid(array $values, $indexPrimary = 'ID', $indexParent = 'PARENT') {
        $aux = array();
        foreach ($values as $item) {
            array_push($aux, $item[$indexPrimary]);
        }
        foreach ($values as &$item) {
            if (!in_array($item[$indexParent], $aux)) {
                $item[$indexParent] = null;
            }
        }
        return $values;
    }

    public static function getValueInFilterDataSourceByName(array &$filters,$nameFilter,$remove = false) {
        foreach ($filters as $key => $filter) {
            $name = $filter['columnName'];
            $values = $filter['value'];
            if($name == $nameFilter ){
                if($remove){
                    unset($filters[$key]);
                }
                return $values;
            }
        }
        return null;
    }

    public static function createQueryFiltered($filters, $tableAlias = null) {
        $filterSQL = '';
        foreach ($filters as $filter) {
            $name = $filter['columnName'];
            $operator = $filter['operator'] == "LIKE_I" ? "LIKE" : $filter['operator'];
            $values = $filter['value'];
            if (($name != '*' && !empty($values)) || $values == 0) {
                if ($operator == 'BETWEEN') {
                    if ($values[0] && $values[1]) {
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')
                                AND TO_DATE( '$values[1] 23:59:59' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[0]) {
                        $operator = '>';
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[1]) {
                        $operator = '<';
                        $values = "TO_DATE( '$values[1] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else {
                        $values = "";
                    }

                    if ($values || ($values && ($values[0] || $values[1]))) {
                        if (!empty($tableAlias)) {
                            $filterSQL .= "AND $tableAlias.$name $operator $values";
                        } else {
                            $filterSQL .= "AND $name $operator $values";
                        }
                    }
                } else {
                    if (is_array($values) && $operator != 'MAPPED_LIKE_ALL') {
                        $stringValues = "('" . implode($values, "','") . "')";
                        $values = $stringValues;
                    }
                    if ($operator == 'LIKE_ALL') {
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                        } else {
                            $values = "UPPER('" . $values . "')";
                        }

                        $columnList = explode('|', $name);
                        $likes = array();
                        foreach ($columnList as $column) {
                            $likes[] = "UPPER($column) LIKE $values";
                        }
                        $filterLikeAll = implode($likes, ' OR ');
                        $filterSQL .= "AND ( $filterLikeAll )";

                    } else if ($operator == 'LIKE') {
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($tableAlias.$name)";
                        } else {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($name)";
                        }
                        $filterSQL .= "AND $name $operator $values";
                    } else if ($operator == 'MAPPED_LIKE_ALL') {
                        foreach($values[0] as $index => $mappedLikeAllValues) {
                            if($index == '_ALL') {
                                if (!empty($tableAlias)) {
                                    $values = "UPPER('" . $mappedLikeAllValues . "')";
                                } else {
                                    $values = "UPPER('" . $mappedLikeAllValues . "')";
                                }

                                $columnList = explode('|', explode('#', $name)[0]);
                                $likes = array();
                                foreach ($columnList as $column) {
                                    $likes[] = "UPPER($column) LIKE $values";
                                }
                                $filterLikeAll = implode($likes, ' OR ');
                                $filterSQL .= " AND ( $filterLikeAll )";
                            } else {
                                $stringValues = "('" . implode($mappedLikeAllValues, "','") . "')";
                                if (!empty($tableAlias)) {
                                    $filterSQL .= " AND $tableAlias.$index IN $stringValues ";
                                } else {
                                    $filterSQL .= " AND $index IN $stringValues ";
                                }
                            }
                        }
                    } else {
                        if (!empty($tableAlias)) {
                            $filterSQL .= "AND $tableAlias.$name $operator $values";
                        } else {
                            $filterSQL .= "AND $name $operator $values";
                        }
                    }
                }
            }
        }
        return $filterSQL;
    }

    public static function validateEmail($email) {
        $emailRegex = "/^[a-zA-Z0-9.!#$%&'*+\/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*$/";
        return preg_match($emailRegex, $email);
    }

    public static function validateNameLogin($name) {
        $nameRegex = "/^[a-zA-ZàáâäãåąčćęèéêëėįìíîïłńòóôöõøùúûüųūÿýżźñçčšžÀÁÂÄÃÅĄĆČĖĘÈÉÊËÌÍÎÏĮŁŃÒÓÔÖÕØÙÚÛÜŲŪŸÝŻŹÑßÇŒÆČŠŽ∂ð ,.'-]+$/";
        return preg_match($nameRegex, $name);
    }

    public static function validateDocumentNumber($documentNumber) {
        if ($documentNumber) {
            return preg_replace("/[^0-9]/", "", $documentNumber);
        }
        return $documentNumber;
    }
} 